<?php

function edstep_course_add_page_view() {
  global $user;

  edstep_authorize();

  drupal_set_title(t('Add course from EdStep'), PASS_THROUGH);

  $query = db_select('edstep_course', 'ec');
  $query->addField('ec', 'course_id');
  $query->condition('ec.uid', $user->uid);
  $added_edstep_course_ids = $query->execute()->fetchCol();

  $client = edstep_api_client();

  $remote_courses = $client->courses();

  $edstep_courses = [];

  foreach($remote_courses as $remote_course) {
    if(in_array($remote_course->id, $added_edstep_course_ids)) {
      continue;
    }
    $edstep_course = edstep_course_create(array(
      'remote' => $remote_course,
    ));
    $edstep_courses[] = $edstep_course;
  }

  usort($edstep_courses, function($edstep_course) {
    return $edstep_course->title;
  });

  foreach($edstep_courses as $edstep_course) {
    $build[] = edstep_course_view($edstep_course, 'manage');
  }

  $no_courses = empty($build);

  if($no_courses) {
    $message = (
      empty($added_edstep_course_ids)
      ? t('You have no courses on EdStep that can be added.')
      : t('You have already added all your courses from EdStep.')
    );
    $build['message'] = array(
      '#theme' => 'html_tag',
      '#tag' => 'div',
      '#value' => $message,
    );
  }

  $build['edstep_link'] = array(
    '#theme' => 'html_tag',
    '#tag' => 'div',
    '#value' => l(t('Create course on EdStep'), variable_get('edstep_host') . '/home/course', array('attributes' => array('target' => '_blank'))),
    '#weight' => (!$no_courses) ? -100 : 100,
  );

  return $build;
}

/**
 * Menu callback: Generates a listing of promoted edstep_courses.
 *
 * @return array
 *   An array in the format expected by drupal_render().
 *
 * @see edstep_course_menu()
 */
function edstep_course_default_page_view() {
  $select = db_select('edstep_course', 'n')
    ->fields('n', array('course_id', 'created'))
    ->orderBy('n.created', 'DESC')
    ->extend('PagerDefault')
    ->limit(variable_get('default_nodes_main', 10));

  $ids = $select->execute()->fetchCol();

  if (!empty($ids)) {
    $edstep_courses = edstep_course_load_multiple($ids);
    $build = edstep_course_view_multiple($edstep_courses);

    $build['pager'] = array(
      '#theme' => 'pager',
      '#weight' => 5,
    );
  }
  else {
    $default_message = '<p>' . t('No EdStep courses have been added yet.') . '</p>';

    $default_links = array();
    if (user_access('add edstep courses')) {
      $default_links[] = l(t('Add new course from EdStep'), 'edstep/course/add');
    }
    if (!empty($default_links)) {
      $default_message .= theme('item_list', array('items' => $default_links));
    }

    $build['default_message'] = array(
      '#markup' => $default_message,
      '#prefix' => '<div id="first-time">',
      '#suffix' => '</div>',
    );
  }
  return $build;
}

/**
 * Menu callback: Displays a single edstep_course.
 *
 * @param $edstep_course
 *   The edstep_course object.
 *
 * @return
 *   A page array suitable for use by drupal_render().
 *
 * @see edstep_course_menu()
 */
function edstep_course_page_view($edstep_course) {
  // If there is a menu link to this edstep_course, the link becomes the last part
  // of the active trail, and the link name becomes the page title.
  // Thus, we must explicitly set the page title to be the edstep_course title.
  drupal_set_title(edstep_course_page_title($edstep_course));
  $uri = entity_uri('edstep_course', $edstep_course);
  // Set the edstep_course path as the canonical URL to prevent duplicate content.
  drupal_add_html_head_link(array('rel' => 'canonical', 'href' => url($uri['path'], $uri['options'])), TRUE);
  // Set the non-aliased path as a default shortlink.
  drupal_add_html_head_link(array('rel' => 'shortlink', 'href' => url($uri['path'], array_merge($uri['options'], array('alias' => TRUE)))), TRUE);
  return edstep_course_show($edstep_course);
}

function edstep_oauth_return_page_view() {
  $params = drupal_get_query_parameters();
  if(!empty($params['error']) && $params['error'] == 'aborted_login') {
    drupal_goto();
  }

  $client = edstep_api_client();
  $provider = $client->getProvider();

  $state = $_SESSION['edstep_oauth_state'];

  if(!$provider->checkState($state)) {
    drupal_access_denied();
  }

  try {
    $accessToken = $client->fetchAccessToken()->getAccessToken();
    edstep_store_access_token($accessToken);
  }
  catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {
    watchdog('edstep', 'Could not authenticate user %uid', array(
      '%uid' => $GLOBALS['user']->uid,
    ), 'error');
    drupal_set_message(t('Could not authenticate against EdStep server. Go back and try again.'), 'error');
    return '';
  }

  try {
    $response = $client->get('test');
  } catch(GuzzleHttp\Exception\ServerException $e) {
    $message = json_decode($e->getResponse()->getBody(), true);
    if($message['error'] == 'No EdStep user') {
      drupal_goto('edstep/oauth/terms-and-conditions', array('query' => array(drupal_get_query_parameters())));
    }
    watchdog('edstep', 'Unexpected error in test request for user %uid', array(
      '%uid' => $GLOBALS['user']->uid,
    ), 'error');
    drupal_set_message(t('Could not authenticate against EdStep server. Go back and try again.'), 'error');
    return '';
  }
  drupal_goto($state['destination']);
}

/**
 * Form constructor for the edstep_course deletion confirmation form.
 *
 * @see edstep_course_delete_confirm_submit()
 */
function edstep_course_delete_confirm($form, &$form_state, $edstep_course) {
  $form['#edstep_course'] = $edstep_course;
  // Always provide entity id in the same form key as in the entity edit form.
  $form['course_id'] = array('#type' => 'value', '#value' => $edstep_course->course_id);
  return confirm_form($form,
    t('Are you sure you want to remove the course %title from this site?', array('%title' => $edstep_course->title)),
    'edstep_course/' . $edstep_course->course_id,
    t('The course will only be removed from this site, not from EdStep. You can add it again later.'),
    t('Remove'),
    t('Cancel')
  );
}

/**
 * Executes edstep_course deletion.
 *
 * @see edstep_course_delete_confirm()
 */
function edstep_course_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $edstep_course = edstep_course_load($form_state['values']['course_id']);
    edstep_course_delete($form_state['values']['course_id']);
    cache_clear_all();
    watchdog('edstep', 'EdStep course: deleted %title.', array('%title' => $edstep_course->title));
    drupal_set_message(t('The course %title has been removed from this site.', array('%title' => $edstep_course->title)));
  }

  $form_state['redirect'] = '<front>';
}

function edstep_my_courses_page_view($filter) {
  edstep_authorize();
  drupal_set_title(t("My $filter courses"));

  $client = edstep_api_client();
  $remote_courses = $client->getCurrentUser()->courses($filter);

  $remote_ids = array();
  foreach($remote_courses as $remote_course) {
    $remote_ids[] = $remote_course->id;
  }

  $ids = array();
  if (!empty($remote_ids)) {
    $select = db_select('edstep_course', 'n')
      ->fields('n', array('course_id', 'created'))
      ->condition('course_id', $remote_ids, 'IN')
      ->orderBy('n.created', 'DESC')
      ->extend('PagerDefault')
      ->limit(variable_get('default_nodes_main', 10));
    $ids = $select->execute()->fetchCol();
  }

  if (!empty($ids)) {
    $edstep_courses = edstep_course_load_multiple($ids);
    $build = edstep_course_view_multiple($edstep_courses);

    $build['pager'] = array(
      '#theme' => 'pager',
      '#weight' => 5,
    );
  }
  else {
    $default_links = array();
    switch($filter) {
      case 'ongoing':
        $default_message = '<p>' . t('You are not taking any courses yet.') . '</p>';
        $default_links[] = l(t('View available courses'), 'edstep/course');
        break;
      case 'completed':
        $default_message = '<p>' . t('You have not completed any courses yet.') . '</p>';
        break;
    }
    if(!empty($default_links)) {
      $default_message .= theme('item_list', array('items' => $default_links));
    }
    $build['default_message'] = array(
      '#markup' => $default_message,
      '#prefix' => '<div id="first-time">',
      '#suffix' => '</div>',
    );
  }

  return $build;
}

function edstep_course_activity_page_view($edstep_course, $section_id, $activity_id) {
  $edstep_course->remote->completed();
  $client = edstep_api_client();
  try {
    $activity = $client->course($edstep_course->course_id)->section($section_id)->activity($activity_id);
    drupal_set_title($activity->title);
    return array(
      '#theme' => 'edstep_course_activity',
      '#edstep_course' => $edstep_course,
      '#activity' => $activity,
      '#attached' => array(
        'js' => array(
          array(
            'data' => drupal_get_path('module', 'edstep') . '/scripts/edstep.login.js',
            'type' => 'file',
          ),
          array(
            'data' => array(
              'edstep' => array(
                'host' => variable_get('edstep_host', ''),
                'access_token' => $client->getAccessToken()->getToken(),
                'path' => edstep_course_activity_resource_url(
                  $edstep_course,
                  $activity->module_id,
                  $activity->id
                ),
              ),
            ),
            'type' => 'setting',
          ),
        ),
      ),
    );
  } catch(Exception $e) {
    drupal_not_found();
  }

}

function edstep_terms_and_conditions_page_view() {
  $build['content'] = array(
    '#theme' => 'html_tag',
    '#tag' => 'iframe',
    '#attributes' => array(
      'id' => 'terms_and_conditions',
      'src' => variable_get('edstep_host') . '/home',
      'width' => '100%',
      'height' => '600px',
      'allowfullscreen' => 'true',
      'frameborder' => 0,
    ),
    '#value' => '',
  );
  $build['#attached']['js'][] = array(
    'data' => drupal_get_path('module', 'edstep') . '/scripts/edstep.terms_and_conditions.js',
    'type' => 'file',
  );

  drupal_add_js(array('edstep' => array('query' => http_build_query(drupal_get_query_parameters()))), 'setting');
  return $build;
}

function edstep_terms_and_conditions_ping() {
  $out = array('success' => true);
  $client = edstep_api_client();
  try {
    $response = $client->get('test');
  } catch(GuzzleHttp\Exception\ServerException $e) {
    $message = json_decode($e->getResponse()->getBody(), true);
    if($message['error'] == 'No EdStep user') {
      $out['success'] = false;
    }
  }
  echo json_encode($out);
}

function edstep_course_result_page_view($edstep_course) {
  $completed = $edstep_course->remote->completed();
  if(!$completed) {
    drupal_goto(edstep_course_continue_uri($edstep_course)['path']);
  }
  drupal_set_title('Congratulations');
  return t('You have completed the course %title.', array('%title' => $edstep_course->title));
}
